import path from "path";
import GppCompiler from "../Compiler/GppCompiler.js";
import printDebug from "../Tools/DebugPrint.js";
import setGetDefaultCompiler from "../Tools/SetGetDefaultCompiler.js";
import HaveCompiler from "./interface/HaveCompiler.js";
import Optimize from "./interface/Optimize.js";
import SourceFiles from "./interface/SourceFiles.js";
import Project from "./Project.js";
import { getNmakePath } from "../Tools/NmakePath.js";
import { getNmakeDir } from "../Tools/NmakeDir.js";
import { getBuildDir } from "../Tools/BuildDir.js";
import printErrorOrDebug from "../Tools/PrintErrorOrDebug.js";
import protectPath from "../Tools/ProtectPath.js";

export default class CppProject implements Project , SourceFiles,HaveCompiler,Optimize {
    name: string;
    compiler: GppCompiler;
    sourceFilesPath: string[]=[];
    optimize: "fast" | "fastest" | "normal" = "fast"
    objs:string[]=[]
    optimizeToBuildString = ():string=>{
        let ret = ""
        switch (this.optimize) {
            case "fast":
                ret="-O2"
                break;
            case "fastest":
                ret="-O3"
                break
            case "normal":
                ret="-O1"
                break
            default:
                printErrorOrDebug("cannot find optimize")
                break;
        }
        return ret
    };
    typeName: string = "cpp";
    addFiles = (...files: string[]):CppProject =>{
        printDebug("adding source files "+files)
        files.forEach(v=>{
            this.sourceFilesPath.push(
                path.join(getNmakeDir(),v)
            )
        });
        printDebug("all files in "+this.name+" :"+this.sourceFilesPath)
        return this
    };

    constructor(name:string)
    constructor(name:string,compiler:GppCompiler)

    constructor(name:string,compiler?:GppCompiler){
        printDebug(`adding new ${this.typeName} project`)
        if(compiler){
            this.compiler = compiler
        }else{
            this.compiler = setGetDefaultCompiler("g++",()=>{return new GppCompiler()})
        }
        this.name=name
    }
}